/* This class encapsulates common gates features and behavoirs.
   Any gate has one output connected to a wire.
   Any gate has its delay.
   Each gate has a reference to a Simulator objeto to
   register future events. 
   The logic of each gate must be defined in a
   specific inherited class */ 

public abstract class Gate implements UpdateListener, InSignal, OutSignal{
  private final int id;  // id for a gate within its class
  private float delay;
  private boolean output;
  private Wire outWire = null;
  private Simulator simulator;
  
  protected Gate (float d, int nextId, Simulator s){
    simulator = s;
    delay = d;
    id = nextId;
  }
  public abstract boolean computeOutput();
    
  // UpdateListener interface
  public void update(){
     boolean newValue = computeOutput();
     if (output != newValue) {
        output = newValue;
        outWire.notifyChange();
     }
  }
  // end UpdateListener

  // InSignal internface
  public void inputChanged() {
     if (output != computeOutput()) {
       ChangeEvent e = new ChangeEvent(this, simulator.getTime()+delay);
       simulator.add(e);
     }
  }
  // end InSignal
  
  // OutSignal interface
  public void connectOutput(Wire w) {
    outWire = w;
    w.connectFrom(this);
  }
  public boolean getOutput() {
     return output;
  }  
  public String getDescription () {
     return getClass().getSimpleName()+"_#"+id;
  }
  // end OutSignal
} 